"use client";

import { Icons } from "@/components/icons";
import { Button } from "@/components/ui/button";
import {
    DropdownMenu,
    DropdownMenuContent,
    DropdownMenuGroup,
    DropdownMenuItem,
    DropdownMenuTrigger,
} from "@/components/ui/dropdown-menu";
import { toast } from "@/components/ui/use-toast";
import { env } from "@/env.mjs";
import { useSession } from "next-auth/react";
import { useRef, useState } from "react";
import { IGweet } from "../types";
import { DeleteGweetModal } from "./delete-gweet-modal";

export const GweetOptions = ({ gweet }: { gweet: IGweet }) => {
    const { data: session } = useSession();

    const [dropdownOpen, setDropdownOpen] = useState<boolean>(false);
    const [hasOpenDialog, setHasOpenDialog] = useState<boolean>(false);
    const dropdownTriggerRef = useRef<HTMLButtonElement | null>(null);
    const focusRef = useRef<HTMLButtonElement | null>(null);

    function handleDialogItemSelect() {
        focusRef.current = dropdownTriggerRef.current;
    }

    function handleDialogItemOpenChange(open: boolean) {
        setHasOpenDialog(open);
        if (open === false) {
            setDropdownOpen(false);
        }
    }

    const url = `${env.NEXT_PUBLIC_APP_URL}/status/${gweet?.id}`;

    return (
        <DropdownMenu open={dropdownOpen} onOpenChange={setDropdownOpen}>
            <DropdownMenuTrigger asChild>
                <div className="h-5 flex items-center">
                    <Button variant="ghost" size="option" ref={dropdownTriggerRef}>
                        <Icons.moreOptions />
                    </Button>
                </div>
            </DropdownMenuTrigger>
            <DropdownMenuContent
                align="end"
                className="font-bold cursor-pointer"
                hidden={hasOpenDialog}
                onClick={(event) => {
                    event.stopPropagation();
                }}
                onCloseAutoFocus={(event) => {
                    if (focusRef.current) {
                        focusRef.current.focus();
                        focusRef.current = null;
                        event.preventDefault();
                    }
                }}>
                <DropdownMenuGroup>
                    {gweet.author.id === session?.user?.id && (
                        <DeleteGweetModal
                            gweet={gweet}
                            props={{
                                triggerChildren: (
                                    <div className="text-red-600 flex">
                                        <Icons.trash className="mr-2 h-4 w-4 " />
                                        <span>Delete</span>
                                    </div>
                                ),
                                onSelect: handleDialogItemSelect,
                                onOpenChange: handleDialogItemOpenChange,
                            }}
                            forwardedRef={focusRef}
                        />
                    )}
                    <DropdownMenuItem
                        onClick={() => {
                            navigator.clipboard.writeText(url);
                            return toast({
                                description: "Copied link to gweet.",
                            })
                        }}>
                        <Icons.link className="mr-2 h-4 w-4 " />
                        <span>Copy link to gweet</span>
                    </DropdownMenuItem>
                </DropdownMenuGroup>
            </DropdownMenuContent>
        </DropdownMenu>
    );
};